﻿using System;
using System.Windows;
using System.Collections.Generic;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls;

namespace HierarchicalGridViewSerialization
{
    public class GridViewHelper : DependencyObject
    {
        public static bool GetIsExpandWatcherActive(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsExpandWatcherActiveProperty);
        }
        public static void SetIsExpandWatcherActive(DependencyObject obj, bool value)
        {
            obj.SetValue(IsExpandWatcherActiveProperty, value);
        }

        public static readonly DependencyProperty IsExpandWatcherActiveProperty =
            DependencyProperty.RegisterAttached("IsExpandWatcherActive", typeof(bool), typeof(GridViewHelper), new PropertyMetadata(false, OnIsExpandWatcherActivePropertyChanged));

        private static void OnIsExpandWatcherActivePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs args)
        {
            RadGridView gridView = d as RadGridView;
            if (gridView != null)
            {
                if ((bool)args.NewValue)
                {
                    GridViewHelper.SetWatcher(gridView, new GridViewSelectionWatcher(gridView));
                }
                else
                {
                    GridViewHelper.SetWatcher(gridView, null);
                }
            }
        }

        public static GridViewSelectionWatcher GetWatcher(DependencyObject obj)
        {
            return (GridViewSelectionWatcher)obj.GetValue(WatcherProperty);
        }
        public static void SetWatcher(DependencyObject obj, GridViewSelectionWatcher value)
        {
            obj.SetValue(WatcherProperty, value);
        }

        public static readonly DependencyProperty WatcherProperty =
            DependencyProperty.RegisterAttached("Watcher", typeof(GridViewSelectionWatcher), typeof(GridViewHelper), new PropertyMetadata(null, OnWatcherPropertyChanged));

        private static void OnWatcherPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs args)
        {
            if (args.NewValue == null && args.OldValue != null)
            {
                GridViewSelectionWatcher watcher = args.OldValue as GridViewSelectionWatcher;
                watcher.DetachEvents();
            }
        }
    }

    public class GridViewSelectionWatcher
    {
        private readonly RadGridView gridView;

        public GridViewSelectionWatcher(RadGridView gridView)
        {
            this.gridView = gridView;
            this.SelectedRowsIdCollection = new List<int>();
            this.AttachEvents();
        }

        public List<int> SelectedRowsIdCollection { get; private set; }

        private void AttachEvents()
        {
            this.gridView.RowLoaded += OnGridView_RowLoaded;
            this.gridView.SelectionChanged += OnGridViewSelectionChanged;
        }

        void OnGridViewSelectionChanged(object sender, SelectionChangeEventArgs e)
        {
            foreach (var item in e.AddedItems)
            {
                if (item is IGridRowSelectionWatchable)
                {
                    int uniqueId = (item as IGridRowSelectionWatchable).GetUniqueId();
                    if (!this.SelectedRowsIdCollection.Contains(uniqueId))
                        this.SelectedRowsIdCollection.Add(uniqueId);
                }
            }
            foreach (var item in e.RemovedItems)
            {
                if (item is IGridRowSelectionWatchable)
                {
                    int uniqueId = (item as IGridRowSelectionWatchable).GetUniqueId();
                    if (this.SelectedRowsIdCollection.Contains(uniqueId))
                        this.SelectedRowsIdCollection.Remove(uniqueId);
                }
            }
        }

        internal void DetachEvents()
        {
            this.gridView.RowLoaded -= OnGridView_RowLoaded;
            this.gridView.SelectionChanged -= OnGridViewSelectionChanged;
        }

        private void OnGridView_RowLoaded(object sender, RowLoadedEventArgs e)
        {
            var watcher = GridViewHelper.GetWatcher(this.gridView);
            if (e.DataElement != null && e.DataElement is IGridRowSelectionWatchable && watcher != null && watcher.SelectedRowsIdCollection.Count != 0)
            {
                int rowId = (e.DataElement as IGridRowSelectionWatchable).GetUniqueId();
                foreach (int id in watcher.SelectedRowsIdCollection)
                {
                    if (id == rowId)
                    {
                        (e.Row as GridViewRow).IsSelected = true;
                    }
                }
            }
        }
    }

    public interface IGridRowSelectionWatchable
    {
        int GetUniqueId();
    }
}
